
#include <iostream>
#include <cstring>
#include <cerrno>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>

using namespace std;


const char* DeviceFile = "/dev/ttyS0";
const speed_t BaudRate = B9600;
const tcflag_t InputFlags = ICRNL;
const tcflag_t OutputFlags = 0;
const tcflag_t ControlFlags = PARENB | CS7 | CLOCAL | CREAD;
const tcflag_t LocalFlags = ICANON;


static struct termios _OldTerminal, _Terminal;
static int _File;


int Initialize(void) 
{
    _File = open(DeviceFile, O_RDWR | O_NDELAY);
    if (_File < 0) {
	cerr << "ERROR: unable to open " << DeviceFile << ": ";
	cerr << strerror(errno) << endl;
	return -1;
    }

    if (tcgetattr(_File, &_OldTerminal) < 0) {
	cerr << "ERROR: unable to get terminal attributes: ";
	cerr << strerror(errno) << endl;
	return -1;
    }
    
    memset(&_Terminal, 0, sizeof(_Terminal));

    _Terminal.c_iflag = InputFlags;
    _Terminal.c_oflag = OutputFlags;
    _Terminal.c_cflag = BaudRate | ControlFlags;
    _Terminal.c_lflag = LocalFlags;
    
    _Terminal.c_cc[VMIN] = 1;
    _Terminal.c_cc[VTIME] = 0;

    tcflush(_File, TCIFLUSH);

    if (tcsetattr(_File, TCSANOW, &_Terminal) < 0) {
	cerr << "ERROR: unable to set terminal attributes: ";
	cerr << strerror(errno) << endl;
	return -1;
    }

    return _File;
}


void Finalize(void)
{
    tcsetattr(_File, TCSANOW, &_OldTerminal);
    close(_File);
}


int main(void)
{
    char Line[1024];
    int LineSize;

    if (Initialize() < 0) {
	return -1;
    }

    while (1) {
	if ((LineSize = read(_File, Line, sizeof(Line)-1)) > 0) {
	    Line[LineSize] = '\0';
	    if (strchr(Line, '\n')) {
		*strchr(Line, '\n') = '\0';
	    }
            if (Line[0] == '\0') {
                continue;
            }

	    cout << Line << endl;
	}
    }

    Finalize();

    return 0;
}
